{
  "type": "record",
  "name": "InviteToken",
  "namespace": "com.linkedin.pegasus2avro.identity",
  "doc": "Aspect used to store the token needed to invite native DataHub users",
  "fields": [
    {
      "name": "token",
      "type": "string",
      "doc": "The encrypted invite token."
    }
  ],
  "Aspect": {
    "name": "inviteToken"
  }
}